﻿//////////////////////////////////////////////
// ComboBox.h
//
//////////////////////////////////////////////

/// Defines / macros -------------------------

#pragma once

/// Includes ---------------------------------

// nkWinUi
#include "Component.h"

#include "../Dll/DllDefines.h"

#include "../Inputs/CallbackRepository.h"

// Standard
#include <map>
#include <string>

/// Class ------------------------------------

namespace nkWinUi
{
	class DLL_WINUI_EXPORT ComboBox : public Component
	{
		public :

			// Entry possible
			class DLL_WINUI_EXPORT Entry final
			{
				private :

					// Texte
					std::string _label ;
					// Données attachées
					void* _attachedData ;

				public :

					// Constructeur, destructeur
					Entry () ;
					~Entry () ;

					// Getters
					nkMemory::StringView getLabel () const ;
					void* getAttachedData () const ;

					// Setters
					void setLabel (const nkMemory::StringView& label) ;
					void setAttachedData (void* data) ;
			} ;

		protected :
		
			// Attributs
			// La liste des entrées
			std::map<std::string, Entry*> _memory ;
			std::map<void*, Entry*> _memoryPerData ;

			// Entrée sélectionnée
			Entry* _selectedEntry ;

			// Callbacks
			comboSelectionCallback _callback ;
		
		public :
		
			// Constructeur, destructeur
			ComboBox () ;
			~ComboBox () ;

			// Getters
			Entry* getEntryPerLabel (const nkMemory::StringView& label) const ;
			Entry* getEntryPerData (void* data) const ;
			virtual Entry* getCurrentSelection () const ;
			comboSelectionCallback getSelectionSpecializedCallback () const ;

			// Setters
			void setSelectionSpecializedCallback (comboSelectionCallback value) ;
			void setSelectedEntryPerLabel (const nkMemory::StringView& label) ;
			void setSelectedEntryPerData (void* data) ;
			virtual void setSelectedEntry (Entry* selected) ;

			// Entries
			virtual Entry* insertEntry (const nkMemory::StringView& label, void* data) ;
			void setEntryData (const nkMemory::StringView& label, void* newData) ;
			virtual void* deleteEntry (const nkMemory::StringView& label) ;
			virtual void clearEntries () ;

			// Pour l'export
			virtual void exportIntrospection (nkExport::Node* rootNode) override ;
			// Import
			virtual void importClassFromTree (nkExport::Node* rootNode) override ;
	} ;
}