﻿//////////////////////////////////////////////
// CallbackRepository.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward declarations ---------------------

namespace nkMaths
{
	class Point ;
}

namespace nkWinUi
{
	class Button ;
	class CheckBox ;
	class ComboBox ;
	class Component ;
	class EditBox ;
	class ListBox ;
	class TabWindow ;
	class TreeView ;
	
	struct CharEventData ;
	struct KeyEventData ;
}

/// Includes ---------------------------------

// nkWinUi
#include "ClickEventData.h"
#include "InputCodes.h"

// Standards
#include <functional>

/// Repo -------------------------------------

namespace nkWinUi
{
	// Les events souris
	using mouseMoveCallback = std::function<bool (Component* caller, int absX, int absY, int relX, int relY, int moveX, int moveY)> ;
	using mouseClickCallback = std::function<bool (Component* caller, const ClickEventData& eventData)> ;
	using mouseLeaveCallback = std::function<bool (Component* caller, const nkMaths::Point& absCoords)> ;
	// Les events claviers
	using keyDownCallback = std::function<bool (Component* caller, KEY_CODE keyCode, bool autoRepeated)> ;
	using keyUpCallback = std::function<bool (Component* caller, KEY_CODE keyCode)> ;
	// Event menu bar
	// Le callback en cas de click sur menu
	using menuBarCallback = std::function<bool (Component* caller, int index)> ;
	// Sur bouton
	using buttonClickCallback = std::function<bool (Button* caller)> ;
	// Check box
	using checkBoxCallback = std::function<bool (CheckBox* caller)> ;
	// Sélection Tree
	using treeSelectionCallback = std::function<bool (TreeView* caller, void* itemData)> ;
	// Sélection ListBox
	using listBoxSelectionCallback = std::function<bool (ListBox* caller)> ;
	// Selection ComboBox
	using comboSelectionCallback = std::function<bool (ComboBox* caller)> ;
	// Editbox avec touche
	using editBoxCharCallback = std::function<bool (EditBox* caller, const CharEventData& key)> ;
	using editBoxKeyCallback = std::function<bool (EditBox* caller, const KeyEventData& key)> ;
	// Au moment de quitter
	using closeCallback = std::function<bool (Component* caller)> ;
	// Resize de window
	using resizeCallback = std::function<bool (Component* caller)> ;
	// Lors d'une demande de ContextMenu
	using contextMenuCreationCallback = std::function<bool (Component* caller, int absX, int absY)> ;
}